import { query } from '@/modules/query';
import { QueryMark, Mark } from '@/query/query-mark/query-mark';
import { useEffect, useRef, useState, useCallback } from 'react';
import { Tooltip } from '@/components/a/tooltip';
export const queryMark = new QueryMark({ query: query, markType: 'md' });
import Fuse from 'fuse.js';
import { debounce } from 'lodash-es';
import { Plus, SquareArrowOutUpRight } from 'lucide-react';
import { Pencil, Trash2 } from 'lucide-react';
import { Button } from '@/components/a/button';
import { Confirm } from '@/components/a/confirm';
import { toast } from 'react-toastify';
import { ToastProvider } from '@/modules/toast/Provider';
import { Modal } from '@/components/a/modal';
import { useForm, Controller } from 'react-hook-form'; // 添加此行

export const App = () => {
  return (
    <ToastProvider>
      <AiMark />
    </ToastProvider>
  );
};
const AiMark = () => {
  const [list, setList] = useState<Mark[]>([]);
  const [searchTerm, setSearchTerm] = useState('');
  const [searchResults, setSearchResults] = useState<Mark[]>([]);

  const fuseRef = useRef<Fuse<Mark>>(null);

  const [open, setOpen] = useState(false);
  const [formData, setFormData] = useState<Mark>();
  const onOpenEdit = (data: Mark) => {
    setOpen(true);
    setFormData(data);
  };
  // 实际执行搜索的函数
  const performSearch = (term: string) => {
    if (!term.trim()) {
      setSearchResults([]);
      return;
    }

    if (fuseRef.current) {
      const results = fuseRef.current.search(term);
      setSearchResults(results.map((result) => result.item));
    }
  };

  // 使用 useRef 和 useCallback 创建防抖搜索函数，确保它在组件生命周期中只创建一次
  const debouncedSearchRef = useRef(
    debounce((term: string) => {
      performSearch(term);
    }, 1000), // 300毫秒的防抖延迟
  );

  useEffect(() => {
    init();

    // 组件卸载时取消待执行的防抖函数
    return () => {
      debouncedSearchRef.current.cancel();
    };
  }, []);

  const init = async () => {
    const res = await queryMark.getMarkList();
    if (res.code === 200) {
      const list = res.data?.list || [];
      setList(list!);

      const fuse = new Fuse(list, {
        keys: ['title', 'description', 'tags', 'summary'],
      });
      fuseRef.current = fuse;
      const result = fuse.search('苏生不惑的博客');
      console.log(result);
    }
  };

  const handleSearch = (term: string) => {
    setSearchTerm(term); // 立即更新输入框的值
    debouncedSearchRef.current(term); // 使用防抖函数进行搜索
  };

  const onOpen = (item: Mark) => {
    const link = item.link;
    window.open(link);
  };
  const refresh = () => {
    init();
  };
  // 确定要显示的列表：有搜索内容时显示搜索结果，否则显示全部
  const displayList = searchTerm.trim() ? searchResults : list;

  return (
    <div className='w-full h-full overflow-auto scrollbar'>
      <div className='sticky top-0 p-4 bg-white z-10 shadow-sm flex gap-2'>
        <Button
          size='icon'
          onClick={() => {
            setFormData(undefined);
            setOpen(true);
          }}>
          <Plus />
        </Button>
        <input
          type='text'
          value={searchTerm}
          onChange={(e) => handleSearch(e.target.value)}
          placeholder='搜索书签...'
          className='w-full p-2 border rounded-md focus:outline-none focus:ring-2 focus:ring-blue-300'
        />
      </div>
      <div className='p-4'>
        {displayList.length === 0 && searchTerm.trim() ? (
          <div className='text-center text-gray-500 my-8'>没有找到匹配的结果</div>
        ) : (
          displayList.map((item) => {
            return (
              <div key={item.id} className='p-4 mb-3 border rounded-md hover:shadow-md transition-shadow' title={item.link}>
                <Tooltip title={item.link}>
                  <h3 className='text-lg font-semibold mb-1 flex gap-2 items-center'>
                    {item.title || '-'}

                    <SquareArrowOutUpRight className='h-5 w-5 cursor-pointer' onClick={() => onOpen(item)} />
                  </h3>
                </Tooltip>
                <p className='text-sm text-gray-600 mb-2'>{item.summary}</p>
                <div className='flex flex-wrap gap-1'>
                  {item.tags.map((tag, index) => (
                    <span key={index} className='px-2 py-1 bg-blue-100 text-blue-800 text-xs rounded-md'>
                      {tag}
                    </span>
                  ))}
                </div>
                <Action data={item} onEdit={onOpenEdit} refresh={refresh} />
              </div>
            );
          })
        )}
      </div>
      <Modal open={open} setOpen={setOpen}>
        <MarkForm
          data={formData}
          onCancel={() => setOpen(false)}
          onSuccess={() => {
            setOpen(false);
            init();
          }}
        />
      </Modal>
    </div>
  );
};

export const MarkForm = (props?: { data?: Mark; onSuccess?: () => void; onCancel?: () => void }) => {
  const {
    register,
    handleSubmit,
    control,
    formState: { errors },
  } = useForm<Mark>({
    defaultValues: props?.data || {
      title: '',
      summary: '',
      link: '',
      tags: [],
    },
  });

  const onSubmit = async (data: Mark) => {
    try {
      const res = await queryMark.updateMark(data);

      if (res.code === 200) {
        toast.success(data.id ? '更新成功' : '创建成功');
        props?.onSuccess?.();
      } else {
        toast.error(res?.message || '操作失败');
      }
    } catch (error) {
      toast.error('操作失败');
    }
  };

  return (
    <form onSubmit={handleSubmit(onSubmit)} className='space-y-4 p-2'>
      {props?.data?.id && <input type='hidden' {...register('id')} />}

      <div>
        <label className='block text-sm font-medium mb-1'>标题</label>
        <input
          type='text'
          {...register('title', { required: '标题是必填项' })}
          className='w-full p-2 border rounded-md focus:outline-none focus:ring-2 focus:ring-blue-300'
        />
        {errors.title && <p className='text-red-500 text-xs mt-1'>{errors.title.message}</p>}
      </div>

      <div>
        <label className='block text-sm font-medium mb-1'>链接</label>
        <input
          type='text'
          {...register('link', { required: '链接是必填项' })}
          className='w-full p-2 border rounded-md focus:outline-none focus:ring-2 focus:ring-blue-300'
        />
        {errors.link && <p className='text-red-500 text-xs mt-1'>{errors.link.message}</p>}
      </div>

      <div>
        <label className='block text-sm font-medium mb-1'>摘要</label>
        <textarea {...register('summary')} rows={3} className='w-full p-2 border rounded-md focus:outline-none focus:ring-2 focus:ring-blue-300' />
      </div>

      <div>
        <label className='block text-sm font-medium mb-1'>标签</label>
        <Controller
          name='tags'
          control={control}
          render={({ field }) => (
            <input
              type='text'
              value={field.value?.join(', ') || ''}
              onChange={(e) => {
                const value = e.target.value;
                field.onChange(
                  value
                    .split(',')
                    .map((tag) => tag.trim())
                    .filter((tag) => tag !== ''),
                );
              }}
              className='w-full p-2 border rounded-md focus:outline-none focus:ring-2 focus:ring-blue-300'
              placeholder='用逗号分隔多个标签'
            />
          )}
        />
      </div>

      <div className='flex justify-end space-x-2 mt-4'>
        <Button onClick={props?.onCancel} type='button'>
          取消
        </Button>
        <Button type='submit'>保存</Button>
      </div>
    </form>
  );
};
export const Action = (props: { data: Mark; onEdit?: (data: Mark) => any; refresh?: any }) => {
  const onDelete = async () => {
    const res = await queryMark.deleteMark(props.data.id);
    if (res.code === 200) {
      toast.success('删除成功');
      props?.refresh?.();
    } else {
      toast.error(res?.message || '请求失败');
    }
  };
  return (
    <div className='flex gap-2 mt-2'>
      <Button className='flex items-center gap-1 transition' onClick={() => props?.onEdit?.(props.data)} title='编辑'>
        <Pencil className='w-4 h-4' />
        编辑
      </Button>
      <Confirm onOk={onDelete}>
        <Button className='flex items-center gap-1 text-red-600 transition' title='删除'>
          <Trash2 className='w-4 h-4' />
          删除
        </Button>
      </Confirm>
    </div>
  );
};
